#!/usr/bin/perl -w

use Gtk;
use Gtk::GladeXML;

my $selectsound = "/usr/share/sounds/gtk-events/clicked.wav";
my $execsound = "/usr/share/sounds/gtk-events/activate.wav";
my $child = 0;

$cursel = 0;
$numitems = 0;

@help = ();
@exec = ();
@title = ();

sub reap_child
{
  waitpid($child,0);
  $child = 0;
  return 1;
}

sub check_audio
{
  if ($child == 0) {
    if ($child = fork) {
    } else {
      die "cannot fork!\n" unless defined $child;
      exec("play",$selectsound);
    }
  }
  return 1;
}

sub read_file
{
  my $file = shift;
  $list->freeze;
  $list->clear;
  $numitems = 0;
  @title = ();
  @exec = ();
  @help = ();
  if (open(FILE,$file)) {
    while (<FILE>) {
      $numitems++;
      chomp;
      my @items = split /\t/;
      push @title, $items[0];
      push @exec, $items[1];
      if (defined($items[2]) && length($items[2])) {
        $items[2] =~ s/\\n/\n/g;
      } else {
        $items[2] = "";
      }
      push @help, $items[2];
    }
    close(FILE);
  }
  for ($i=0; $i<$numitems; $i++) {
    $list->append($title[$i]);
  }
  $cursel = 0;
  $list->select_row(0,0);
  $apptitle->set_text($title[0]);
  $text->backward_delete($text->get_length);
  if ($numitems) {
    $text->insert(undef,undef,undef,$help[0]);
  } else {
    $text->insert(undef,undef,undef,"No Text\n");
  }
  $list->thaw;
  return 1;
}

sub quit_app
{
  Gtk->exit(0);
  return 0;
}

sub scroll_list
{
  my $dir = shift;
  my $adj = $list->get_vadjustment();
  #my $newval = $adj->get_value + ($dir / $numitems * $adj->upper);
  my $newval = ($cursel / $numitems * $adj->upper) - ($adj->page_size/2);
  if ($newval > ($adj->upper - ($adj->page_size/1))) {
    $newval = $adj->upper - ($adj->page_size/1);
  }
  $adj->set_value($newval);
  $list->set_vadjustment($adj);
}

sub update_display
{
  $list->select_row($cursel,0);
  $text->freeze;
  $text->backward_delete($text->get_length);
  HELPTEST: for (my $i = $cursel; $i >= 0; $i--) {
    if (length($help[$i])) {
      $text->insert(undef,undef,undef,$help[$i]);
      last HELPTEST;
    }
  }
  scroll_list(-1);
  $apptitle->set_text($title[$cursel]);
  check_audio;
  $text->thaw;
}

sub readpad
{
  my $button = <BI>;
  if (!defined($button)) { return 1; }
  chomp $button;
  if (($button eq "Left") || ($button eq "Up")) {
    $list->unselect_row($cursel,0);
    $cursel--;
    if ($cursel < 0) { $cursel = $numitems-1; }
    update_display;
  }
  if (($button eq "Right") || ($button eq "Down")) {
    $list->unselect_row($cursel,0);
    $cursel++;
    if ($cursel >= $numitems) { $cursel = 0; }
    update_display;
  }
  if (($button eq "L1") || ($button eq "R1")) {
    $list->unselect_row($cursel,0);
    $cursel -= 10;
    if ($cursel <= 0) { $cursel = 0; }
    update_display;
  }
  if (($button eq "L2") || ($button eq "R2")) {
    $list->unselect_row($cursel,0);
    $cursel += 10;
    if ($cursel >= $numitems) { $cursel = $numitems - 1; }
    update_display;
  }
  if (($button eq "X") && (length($exec[$cursel]))) {
    if ($exec[$cursel] =~ /menu:(.*)/i) {
      read_file($1);
    } elsif (!(-e "/tmp/nostart")) {
      system("killall readbuttons");
      close(BI);
      system("play",$execsound);
      exec($exec[$cursel]);
    }
  }
  if ($button eq "Triangle") {
    system("killall readbuttons");
    close(BI);
    quit_app;
  }
  return 1;
}

Gtk->init;
Gtk::GladeXML->init;

$gui = new Gtk::GladeXML("launcher.glade");
$gui->handler_connect('gtk_main_quit',\&quit_app);

$window = $gui->get_widget("mainwindow");
$window->signal_connect("destroy", \&quit_app);
$window->signal_connect("delete_event", \&quit_app);

$list = $gui->get_widget("applist");

$text = $gui->get_widget("helptext");
$text->insert(undef,undef,undef,"Config file not found\n");

$list->select_row($cursel,0);

$apptitle = $gui->get_widget("apptitle");

read_file("launcher.cfg");

open(BI,"./readbuttons 2>&1 |") or die "Can't open button reader\n";

Gtk::Gdk->input_add(fileno(BI),['read'],\&readpad);

$SIG{CHLD} = \&reap_child;

Gtk->main;
